@@ -17,11 +17,16 @@
 // Copyright (C) 2005-2007 Jeff Muizelaar <jeff@infidigm.net>
 // Copyright (C) 2005, 2006 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2005 Martin Kretzschmar <martink@gnome.org>
-// Copyright (C) 2005 Albert Astals Cid <aacid@kde.org>
-// Copyright (C) 2006, 2007 Carlos Garcia Campos <carlosgc@gnome.org>
+// Copyright (C) 2005, 2009, 2012, 2013 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006, 2007, 2010, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2007 Koji Otani <sho@bbr.jp>
-// Copyright (C) 2008 Chris Wilson <chris@chris-wilson.co.uk>
+// Copyright (C) 2008, 2009 Chris Wilson <chris@chris-wilson.co.uk>
+// Copyright (C) 2008, 2012 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2009 Darren Kenny <darren.kenny@sun.com>
+// Copyright (C) 2010 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
+// Copyright (C) 2010 Jan Kümmel <jan+freedesktop@snorc.org>
+// Copyright (C) 2012 Hib Eris <hib@hiberis.nl>
+// Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -33,12 +38,15 @@
 #include "config.h"
 #include <string.h>
 #include "CairoFontEngine.h"
-#include "CharCodeToUnicode.h"
+#include "CairoOutputDev.h"
 #include "GlobalParams.h"
 #include <fofi/FoFiTrueType.h>
 #include <fofi/FoFiType1C.h>
 #include "goo/gfile.h"
 #include "Error.h"
+#include "XRef.h"
+#include "Gfx.h"
+#include "Page.h"
 
 #if HAVE_FCNTL_H && HAVE_SYS_MMAN_H && HAVE_SYS_STAT_H
 #include <fcntl.h>
@@ -51,14 +59,106 @@
 #pragma implementation
 #endif
 
-static void fileWrite(void *stream, char *data, int len) {
-  fwrite(data, 1, len, (FILE *)stream);
-}
+#if MULTITHREADED
+#  define fontEngineLocker()   MutexLocker locker(&mutex)
+#else
+#  define fontEngineLocker()
+#endif
 
 //------------------------------------------------------------------------
 // CairoFont
 //------------------------------------------------------------------------
 
+CairoFont::CairoFont(Ref ref,
+		     cairo_font_face_t *cairo_font_face,
+		     int *codeToGID,
+		     Guint codeToGIDLen,
+		     GBool substitute,
+		     GBool printing) : ref(ref),
+				       cairo_font_face(cairo_font_face),
+				       codeToGID(codeToGID),
+				       codeToGIDLen(codeToGIDLen),
+				       substitute(substitute),
+				       printing(printing)      { }
+
+CairoFont::~CairoFont() {
+  cairo_font_face_destroy (cairo_font_face);
+  gfree(codeToGID);
+}
+
+GBool
+CairoFont::matches(Ref &other, GBool printingA) {
+  return (other.num == ref.num && other.gen == ref.gen);
+}
+
+cairo_font_face_t *
+CairoFont::getFontFace(void) {
+  return cairo_font_face;
+}
+
+unsigned long
+CairoFont::getGlyph(CharCode code,
+		    Unicode *u, int uLen) {
+  FT_UInt gid;
+
+  if (codeToGID && code < codeToGIDLen) {
+    gid = (FT_UInt)codeToGID[code];
+  } else {
+    gid = (FT_UInt)code;
+  }
+  return gid;
+}
+
+double
+CairoFont::getSubstitutionCorrection(GfxFont *gfxFont)
+{
+  double w1, w2;
+  CharCode code;
+  char *name;
+
+  // for substituted fonts: adjust the font matrix -- compare the
+  // width of 'm' in the original font and the substituted font
+  if (isSubstitute() && !gfxFont->isCIDFont()) {
+    for (code = 0; code < 256; ++code) {
+      if ((name = ((Gfx8BitFont *)gfxFont)->getCharName(code)) &&
+	  name[0] == 'm' && name[1] == '\0') {
+	break;
+      }
+    }
+    if (code < 256) {
+      w1 = ((Gfx8BitFont *)gfxFont)->getWidth(code);
+      {
+	cairo_matrix_t m;
+	cairo_matrix_init_identity(&m);
+	cairo_font_options_t *options = cairo_font_options_create();
+	cairo_font_options_set_hint_style(options, CAIRO_HINT_STYLE_NONE);
+	cairo_font_options_set_hint_metrics(options, CAIRO_HINT_METRICS_OFF);
+	cairo_scaled_font_t *scaled_font = cairo_scaled_font_create(cairo_font_face, &m, &m, options);
+
+	cairo_text_extents_t extents;
+	cairo_scaled_font_text_extents(scaled_font, "m", &extents);
+
+	cairo_scaled_font_destroy(scaled_font);
+	cairo_font_options_destroy(options);
+	w2 = extents.x_advance;
+      }
+      if (!gfxFont->isSymbolic()) {
+	// if real font is substantially narrower than substituted
+	// font, reduce the font size accordingly
+	if (w1 > 0.01 && w1 < 0.9 * w2) {
+	  w1 /= w2;
+	  return w1;
+	}
+      }
+    }
+  }
+  return 1.0;
+}
+
+//------------------------------------------------------------------------
+// CairoFreeTypeFont
+//------------------------------------------------------------------------
+
 static cairo_user_data_key_t _ft_cairo_key;
 
 static void
@@ -71,14 +171,21 @@ _ft_done_face_uncached (void *closure)
 static GBool
 _ft_new_face_uncached (FT_Library lib,
 		       const char *filename,
+		       char *font_data,
+		       int font_data_len,
 		       FT_Face *face_out,
 		       cairo_font_face_t **font_face_out)
 {
   FT_Face face;
   cairo_font_face_t *font_face;
 
-  if (FT_New_Face (lib, filename, 0, &face))
-    return gFalse;
+  if (font_data == NULL) {
+    if (FT_New_Face (lib, filename, 0, &face))
+      return gFalse;
+  } else {
+    if (FT_New_Memory_Face (lib, (unsigned char *)font_data, font_data_len, 0, &face))
+      return gFalse;
+  }
 
   font_face = cairo_ft_font_face_create_for_ft_face (face,
 							  FT_LOAD_NO_HINTING |
@@ -163,6 +270,8 @@ _ft_done_face (void *closure)
 static GBool
 _ft_new_face (FT_Library lib,
 	      const char *filename,
+	      char *font_data,
+	      int font_data_len,
 	      FT_Face *face_out,
 	      cairo_font_face_t **font_face_out)
 {
@@ -170,37 +279,46 @@ _ft_new_face (FT_Library lib,
   struct stat st;
   struct _ft_face_data tmpl;
 
-  /* if we fail to mmap the file, just pass it to FreeType instead */
-  tmpl.fd = open (filename, O_RDONLY);
-  if (tmpl.fd == -1)
-    return _ft_new_face_uncached (lib, filename, face_out, font_face_out);
+  tmpl.fd = -1;
 
-  if (fstat (tmpl.fd, &st) == -1) {
-    close (tmpl.fd);
-    return _ft_new_face_uncached (lib, filename, face_out, font_face_out);
-  }
+  if (font_data == NULL) {
+    /* if we fail to mmap the file, just pass it to FreeType instead */
+    tmpl.fd = open (filename, O_RDONLY);
+    if (tmpl.fd == -1)
+      return _ft_new_face_uncached (lib, filename, font_data, font_data_len, face_out, font_face_out);
 
-  tmpl.bytes = (unsigned char *) mmap (NULL, st.st_size,
-				       PROT_READ, MAP_PRIVATE,
-				       tmpl.fd, 0);
-  if (tmpl.bytes == MAP_FAILED) {
-    close (tmpl.fd);
-    return _ft_new_face_uncached (lib, filename, face_out, font_face_out);
+    if (fstat (tmpl.fd, &st) == -1) {
+      close (tmpl.fd);
+      return _ft_new_face_uncached (lib, filename, font_data, font_data_len, face_out, font_face_out);
+    }
+
+    tmpl.bytes = (unsigned char *) mmap (NULL, st.st_size,
+					 PROT_READ, MAP_PRIVATE,
+					 tmpl.fd, 0);
+    if (tmpl.bytes == MAP_FAILED) {
+      close (tmpl.fd);
+      return _ft_new_face_uncached (lib, filename, font_data, font_data_len, face_out, font_face_out);
+    }
+    tmpl.size = st.st_size;
+  } else {
+    tmpl.bytes = (unsigned char*) font_data;
+    tmpl.size = font_data_len;
   }
 
   /* check to see if this is a duplicate of any of the currently open fonts */
   tmpl.lib = lib;
-  tmpl.size = st.st_size;
   tmpl.hash = _djb_hash (tmpl.bytes, tmpl.size);
 
   for (l = _ft_open_faces; l; l = l->next) {
     if (_ft_face_data_equal (l, &tmpl)) {
+      if (tmpl.fd != -1) {
 #if defined(__SUNPRO_CC) && defined(__sun) && defined(__SVR4)
-      munmap ((char*)tmpl.bytes, tmpl.size);
+        munmap ((char*)tmpl.bytes, tmpl.size);
 #else
-      munmap (tmpl.bytes, tmpl.size);
+        munmap (tmpl.bytes, tmpl.size);
 #endif
-      close (tmpl.fd);
+        close (tmpl.fd);
+      }
       *face_out = l->face;
       *font_face_out = cairo_font_face_reference (l->font_face);
       return gTrue;
@@ -208,14 +326,19 @@ _ft_new_face (FT_Library lib,
   }
 
   /* not a dup, open and insert into list */
-  if (FT_New_Face (lib, filename, 0, &tmpl.face)) {
+  if (FT_New_Memory_Face (lib,
+			  (FT_Byte *) tmpl.bytes, tmpl.size,
+			  0, &tmpl.face))
+  {
+    if (tmpl.fd != -1) {
 #if defined(__SUNPRO_CC) && defined(__sun) && defined(__SVR4)
-    munmap ((char*)tmpl.bytes, tmpl.size);
+      munmap ((char*)tmpl.bytes, tmpl.size);
 #else
-    munmap (tmpl.bytes, tmpl.size);
+      munmap (tmpl.bytes, tmpl.size);
 #endif
 
-    close (tmpl.fd);
+      close (tmpl.fd);
+    }
     return gFalse;
   }
 
@@ -237,7 +360,7 @@ _ft_new_face (FT_Library lib,
   {
     cairo_font_face_destroy (l->font_face);
     _ft_done_face (l);
-    return NULL;
+    return gFalse;
   }
 
   *face_out = l->face;
@@ -248,14 +371,29 @@ _ft_new_face (FT_Library lib,
 #define _ft_new_face _ft_new_face_uncached
 #endif
 
-CairoFont *CairoFont::create(GfxFont *gfxFont, XRef *xref, FT_Library lib, GBool useCIDs) {
-  Ref embRef;
+CairoFreeTypeFont::CairoFreeTypeFont(Ref ref,
+				     cairo_font_face_t *cairo_font_face,
+				     int *codeToGID,
+				     Guint codeToGIDLen,
+				     GBool substitute) : CairoFont(ref,
+								   cairo_font_face,
+								   codeToGID,
+								   codeToGIDLen,
+								   substitute,
+								   gTrue) { }
+
+CairoFreeTypeFont::~CairoFreeTypeFont() { }
+
+CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
+					     FT_Library lib, GBool useCIDs) {
   Object refObj, strObj;
-  GooString *tmpFileName, *fileName,*tmpFileName2;
-  DisplayFontParam *dfp;
-  FILE *tmpFile;
-  int c, i, n;
+  GooString *fileName;
+  char *fileNameC;
+  char *font_data;
+  int font_data_len;
+  int i, n;
   GfxFontType fontType;
+  GfxFontLoc *fontLoc;
   char **enc;
   char *name;
   FoFiTrueType *ff;
@@ -264,101 +402,82 @@ CairoFont *CairoFont::create(GfxFont *gfxFont, XRef *xref, FT_Library lib, GBool
   FT_Face face;
   cairo_font_face_t *font_face;
 
-  Gushort *codeToGID;
-  int codeToGIDLen;
+  int *codeToGID;
+  Guint codeToGIDLen;
   
-  dfp = NULL;
   codeToGID = NULL;
   codeToGIDLen = 0;
+  font_data = NULL;
+  font_data_len = 0;
+  fileName = NULL;
+  fileNameC = NULL;
 
   GBool substitute = gFalse;
   
   ref = *gfxFont->getID();
   fontType = gfxFont->getType();
 
-  tmpFileName = NULL;
+  if (!(fontLoc = gfxFont->locateFont(xref, gFalse))) {
+    error(errSyntaxError, -1, "Couldn't find a font for '{0:s}'",
+    	gfxFont->getName() ? gfxFont->getName()->getCString()
+	                       : "(unnamed)");
+    goto err2;
+  }
 
-  if (gfxFont->getEmbeddedFontID(&embRef)) {
-    if (!openTempFile(&tmpFileName, &tmpFile, "wb", NULL)) {
-      error(-1, "Couldn't create temporary font file");
+  // embedded font
+  if (fontLoc->locType == gfxFontLocEmbedded) {
+    font_data = gfxFont->readEmbFontFile(xref, &font_data_len);
+    if (NULL == font_data)
       goto err2;
-    }
-    
-    refObj.initRef(embRef.num, embRef.gen);
-    refObj.fetch(xref, &strObj);
-    refObj.free();
-    if (!strObj.isStream()) {
-      error(-1, "Embedded font object is wrong type");
-      strObj.free();
-      fclose(tmpFile);
-      goto err2;
-    }
-    strObj.streamReset();
-    while ((c = strObj.streamGetChar()) != EOF) {
-      fputc(c, tmpFile);
-    }
-    strObj.streamClose();
-    strObj.free();
-    fclose(tmpFile);
-    fileName = tmpFileName;
-    
-  } else if (!(fileName = gfxFont->getExtFontFile())) {
-    // look for a display font mapping or a substitute font
-    dfp = NULL;
-    if (gfxFont->getName()) {
-      dfp = globalParams->getDisplayFont(gfxFont);
-    }
-    if (!dfp) {
-      error(-1, "Couldn't find a font for '%s'",
-	    gfxFont->getName() ? gfxFont->getName()->getCString()
-	    : "(unnamed)");
-      goto err2;
-    }
-    switch (dfp->kind) {
-    case displayFontT1:
-      fileName = dfp->t1.fileName;
-      fontType = gfxFont->isCIDFont() ? fontCIDType0 : fontType1;
-      break;
-    case displayFontTT:
-      fileName = dfp->tt.fileName;
-      fontType = gfxFont->isCIDFont() ? fontCIDType2 : fontTrueType;
-      break;
-    }
+
+  // external font
+  } else { // gfxFontLocExternal
+    fileName = fontLoc->path;
+    fontType = fontLoc->fontType;
     substitute = gTrue;
   }
 
+  if (fileName != NULL) {
+    fileNameC = fileName->getCString();
+  }
+
   switch (fontType) {
   case fontType1:
   case fontType1C:
-    if (! _ft_new_face (lib, fileName->getCString(), &face, &font_face)) {
-      error(-1, "could not create type1 face");
+  case fontType1COT:
+    if (! _ft_new_face (lib, fileNameC, font_data, font_data_len, &face, &font_face)) {
+      error(errSyntaxError, -1, "could not create type1 face");
       goto err2;
     }
     
     enc = ((Gfx8BitFont *)gfxFont)->getEncoding();
     
-    codeToGID = (Gushort *)gmallocn(256, sizeof(int));
+    codeToGID = (int *)gmallocn(256, sizeof(int));
     codeToGIDLen = 256;
     for (i = 0; i < 256; ++i) {
       codeToGID[i] = 0;
       if ((name = enc[i])) {
-	codeToGID[i] = (Gushort)FT_Get_Name_Index(face, name);
+	codeToGID[i] = FT_Get_Name_Index(face, name);
       }
     }
     break;
-    
   case fontCIDType2:
+  case fontCIDType2OT:
     codeToGID = NULL;
     n = 0;
     if (((GfxCIDFont *)gfxFont)->getCIDToGID()) {
       n = ((GfxCIDFont *)gfxFont)->getCIDToGIDLen();
       if (n) {
-	codeToGID = (Gushort *)gmallocn(n, sizeof(Gushort));
+	codeToGID = (int *)gmallocn(n, sizeof(int));
 	memcpy(codeToGID, ((GfxCIDFont *)gfxFont)->getCIDToGID(),
-		n * sizeof(Gushort));
+		n * sizeof(int));
       }
     } else {
-      ff = FoFiTrueType::load(fileName->getCString());
+      if (font_data != NULL) {
+        ff = FoFiTrueType::make(font_data, font_data_len);
+      } else {
+        ff = FoFiTrueType::load(fileNameC);
+      }
       if (! ff)
 	goto err2;
       codeToGID = ((GfxCIDFont *)gfxFont)->getCodeToGIDMap(ff, &n);
@@ -367,8 +486,13 @@ CairoFont *CairoFont::create(GfxFont *gfxFont, XRef *xref, FT_Library lib, GBool
     codeToGIDLen = n;
     /* Fall through */
   case fontTrueType:
-    if (!(ff = FoFiTrueType::load(fileName->getCString()))) {
-      error(-1, "failed to load truetype font\n");
+    if (font_data != NULL) {
+      ff = FoFiTrueType::make(font_data, font_data_len);
+    } else {
+      ff = FoFiTrueType::load(fileNameC);
+    }
+    if (! ff) {
+      error(errSyntaxError, -1, "failed to load truetype font\n");
       goto err2;
     }
     /* This might be set already for the CIDType2 case */
@@ -376,21 +500,11 @@ CairoFont *CairoFont::create(GfxFont *gfxFont, XRef *xref, FT_Library lib, GBool
       codeToGID = ((Gfx8BitFont *)gfxFont)->getCodeToGIDMap(ff);
       codeToGIDLen = 256;
     }
-    if (!openTempFile(&tmpFileName2, &tmpFile, "wb", NULL)) {
-      delete ff;
-      error(-1, "failed to open truetype tempfile\n");
-      goto err2;
-    }
-    ff->writeTTF(&fileWrite, tmpFile);
-    fclose(tmpFile);
     delete ff;
-
-    if (! _ft_new_face (lib, tmpFileName2->getCString(), &face, &font_face)) {
-      error(-1, "could not create truetype face\n");
+    if (! _ft_new_face (lib, fileNameC, font_data, font_data_len, &face, &font_face)) {
+      error(errSyntaxError, -1, "could not create truetype face\n");
       goto err2;
     }
-    unlink (tmpFileName2->getCString());
-    delete tmpFileName2;
     break;
     
   case fontCIDType0:
@@ -401,125 +515,237 @@ CairoFont *CairoFont::create(GfxFont *gfxFont, XRef *xref, FT_Library lib, GBool
 
     if (!useCIDs)
     {
-      if ((ff1c = FoFiType1C::load(fileName->getCString()))) {
-        codeToGID = ff1c->getCIDToGIDMap(&codeToGIDLen);
+      if (font_data != NULL) {
+        ff1c = FoFiType1C::make(font_data, font_data_len);
+      } else {
+        ff1c = FoFiType1C::load(fileNameC);
+      }
+      if (ff1c) {
+        codeToGID = ff1c->getCIDToGIDMap((int *)&codeToGIDLen);
         delete ff1c;
       }
     }
 
-    if (! _ft_new_face (lib, fileName->getCString(), &face, &font_face)) {
+    if (! _ft_new_face (lib, fileNameC, font_data, font_data_len, &face, &font_face)) {
       gfree(codeToGID);
       codeToGID = NULL;
-      error(-1, "could not create cid face\n");
+      error(errSyntaxError, -1, "could not create cid face\n");
       goto err2;
     }
     break;
     
   default:
-    printf ("font type not handled\n");
+    fprintf (stderr, "font type %d not handled\n", (int)fontType);
     goto err2;
     break;
   }
 
-  // delete the (temporary) font file -- with Unix hard link
-  // semantics, this will remove the last link; otherwise it will
-  // return an error, leaving the file to be deleted later
-  if (fileName == tmpFileName) {
-    unlink (fileName->getCString());
-    delete tmpFileName;
-  }
-
-  return new CairoFont(ref,
-		       font_face, face,
+  delete fontLoc;
+  return new CairoFreeTypeFont(ref,
+		       font_face,
 		       codeToGID, codeToGIDLen,
 		       substitute);
 
  err2:
   /* hmm? */
-  printf ("some font thing failed\n");
+  delete fontLoc;
+  fprintf (stderr, "some font thing failed\n");
   return NULL;
 }
 
-CairoFont::CairoFont(Ref ref, cairo_font_face_t *cairo_font_face, FT_Face face,
-    Gushort *codeToGID, int codeToGIDLen, GBool substitute) : ref(ref), cairo_font_face(cairo_font_face),
-					    face(face), codeToGID(codeToGID),
-					    codeToGIDLen(codeToGIDLen), substitute(substitute) { }
+//------------------------------------------------------------------------
+// CairoType3Font
+//------------------------------------------------------------------------
 
-CairoFont::~CairoFont() {
-  cairo_font_face_destroy (cairo_font_face);
-  gfree(codeToGID);
-}
+static const cairo_user_data_key_t type3_font_key = {0};
 
-GBool
-CairoFont::matches(Ref &other) {
-  return (other.num == ref.num && other.gen == ref.gen);
-}
+typedef struct _type3_font_info {
+  GfxFont *font;
+  PDFDoc *doc;
+  CairoFontEngine *fontEngine;
+  GBool printing;
+  XRef *xref;
+} type3_font_info_t;
 
-cairo_font_face_t *
-CairoFont::getFontFace(void) {
-  return cairo_font_face;
-}
-
-unsigned long
-CairoFont::getGlyph(CharCode code,
-		    Unicode *u, int uLen) {
-  FT_UInt gid;
-
-  if (codeToGID && code < codeToGIDLen) {
-    gid = (FT_UInt)codeToGID[code];
-  } else {
-    gid = (FT_UInt)code;
-  }
-  return gid;
-}
-
-double
-CairoFont::getSubstitutionCorrection(GfxFont *gfxFont)
+static void
+_free_type3_font_info(void *closure)
 {
-  double w1, w2,w3;
-  CharCode code;
+  type3_font_info_t *info = (type3_font_info_t *) closure;
+
+  info->font->decRefCnt();
+  free (info);
+}
+
+static cairo_status_t
+_init_type3_glyph (cairo_scaled_font_t  *scaled_font,
+                   cairo_t              *cr,
+                   cairo_font_extents_t *extents)
+{
+  type3_font_info_t *info;
+  GfxFont *font;
+  double *mat;
+
+  info = (type3_font_info_t *)
+    cairo_font_face_get_user_data (cairo_scaled_font_get_font_face (scaled_font),
+				   &type3_font_key);
+  font = info->font;
+  mat = font->getFontBBox();
+  extents->ascent = mat[3]; /* y2 */
+  extents->descent = -mat[3]; /* -y1 */
+  extents->height = extents->ascent + extents->descent;
+  extents->max_x_advance = mat[2] - mat[1]; /* x2 - x1 */
+  extents->max_y_advance = 0;
+
+  return CAIRO_STATUS_SUCCESS;
+}
+
+static cairo_status_t
+_render_type3_glyph (cairo_scaled_font_t  *scaled_font,
+		     unsigned long         glyph,
+		     cairo_t              *cr,
+		     cairo_text_extents_t *metrics)
+{
+  Dict *charProcs;
+  Object charProc;
+  CairoOutputDev *output_dev;
+  cairo_matrix_t matrix, invert_y_axis;
+  double *mat;
+  double wx, wy;
+  PDFRectangle box;
+  type3_font_info_t *info;
+  GfxFont *font;
+  Dict *resDict;
+  Gfx *gfx;
+
+  info = (type3_font_info_t *)
+    cairo_font_face_get_user_data (cairo_scaled_font_get_font_face (scaled_font),
+				   &type3_font_key);
+
+  font = info->font;
+  resDict = ((Gfx8BitFont *)font)->getResources();
+  charProcs = ((Gfx8BitFont *)(info->font))->getCharProcs();
+  if (!charProcs)
+    return CAIRO_STATUS_USER_FONT_ERROR;
+
+  if ((int)glyph >= charProcs->getLength())
+    return CAIRO_STATUS_USER_FONT_ERROR;
+
+  mat = font->getFontMatrix();
+  matrix.xx = mat[0];
+  matrix.yx = mat[1];
+  matrix.xy = mat[2];
+  matrix.yy = mat[3];
+  matrix.x0 = mat[4];
+  matrix.y0 = mat[5];
+  cairo_matrix_init_scale (&invert_y_axis, 1, -1);
+  cairo_matrix_multiply (&matrix, &matrix, &invert_y_axis);
+  cairo_transform (cr, &matrix);
+
+  output_dev = new CairoOutputDev();
+  output_dev->setCairo(cr);
+  output_dev->setPrinting(info->printing);
+
+  mat = font->getFontBBox();
+  box.x1 = mat[0];
+  box.y1 = mat[1];
+  box.x2 = mat[2];
+  box.y2 = mat[3];
+  gfx = new Gfx(info->doc, output_dev, resDict, &box, NULL);
+  output_dev->startDoc(info->doc, info->fontEngine);
+  output_dev->startPage (1, gfx->getState(), gfx->getXRef());
+  output_dev->setInType3Char(gTrue);
+  gfx->display(charProcs->getVal(glyph, &charProc));
+
+  output_dev->getType3GlyphWidth (&wx, &wy);
+  cairo_matrix_transform_distance (&matrix, &wx, &wy);
+  metrics->x_advance = wx;
+  metrics->y_advance = wy;
+  if (output_dev->hasType3GlyphBBox()) {
+    double *bbox = output_dev->getType3GlyphBBox();
+
+    cairo_matrix_transform_point (&matrix, &bbox[0], &bbox[1]);
+    cairo_matrix_transform_point (&matrix, &bbox[2], &bbox[3]);
+    metrics->x_bearing = bbox[0];
+    metrics->y_bearing = bbox[1];
+    metrics->width = bbox[2] - bbox[0];
+    metrics->height = bbox[3] - bbox[1];
+  }
+
+  delete gfx;
+  delete output_dev;
+  charProc.free();
+
+  return CAIRO_STATUS_SUCCESS;
+}
+
+
+CairoType3Font *CairoType3Font::create(GfxFont *gfxFont, PDFDoc *doc,
+				       CairoFontEngine *fontEngine,
+				       GBool printing, XRef *xref) {
+  Object refObj, strObj;
+  type3_font_info_t *info;
+  cairo_font_face_t *font_face;
+  Ref ref;
+  int *codeToGID;
+  Guint codeToGIDLen;
+  int i, j;
+  char **enc;
+  Dict *charProcs;
   char *name;
 
-  // for substituted fonts: adjust the font matrix -- compare the
-  // width of 'm' in the original font and the substituted font
-  if (isSubstitute() && !gfxFont->isCIDFont()) {
-    for (code = 0; code < 256; ++code) {
-      if ((name = ((Gfx8BitFont *)gfxFont)->getCharName(code)) &&
-	  name[0] == 'm' && name[1] == '\0') {
-	break;
-      }
-    }
-    if (code < 256) {
-      w1 = ((Gfx8BitFont *)gfxFont)->getWidth(code);
-      {
-	cairo_matrix_t m;
-	cairo_matrix_init_identity(&m);
-	cairo_font_options_t *options = cairo_font_options_create();
-	cairo_font_options_set_hint_style(options, CAIRO_HINT_STYLE_NONE);
-	cairo_font_options_set_hint_metrics(options, CAIRO_HINT_METRICS_OFF);
-	cairo_scaled_font_t *scaled_font = cairo_scaled_font_create(cairo_font_face, &m, &m, options);
+  charProcs = ((Gfx8BitFont *)gfxFont)->getCharProcs();
+  info = (type3_font_info_t *) malloc(sizeof(*info));
+  ref = *gfxFont->getID();
+  font_face = cairo_user_font_face_create();
+  cairo_user_font_face_set_init_func (font_face, _init_type3_glyph);
+  cairo_user_font_face_set_render_glyph_func (font_face, _render_type3_glyph);
+  gfxFont->incRefCnt();
+  info->font = gfxFont;
+  info->doc = doc;
+  info->fontEngine = fontEngine;
+  info->printing = printing;
+  info->xref = xref;
 
-	cairo_text_extents_t extents;
-	cairo_scaled_font_text_extents(scaled_font, "m", &extents);
+  cairo_font_face_set_user_data (font_face, &type3_font_key, (void *) info, _free_type3_font_info);
 
-	cairo_scaled_font_destroy(scaled_font);
-	cairo_font_options_destroy(options);
-	w3 = extents.width;
-	w2 = extents.x_advance;
-      }
-      if (!gfxFont->isSymbolic()) {
-	// if real font is substantially narrower than substituted
-	// font, reduce the font size accordingly
-	if (w1 > 0.01 && w1 < 0.9 * w2) {
-	  w1 /= w2;
-	  return w1;
+  enc = ((Gfx8BitFont *)gfxFont)->getEncoding();
+  codeToGID = (int *)gmallocn(256, sizeof(int));
+  codeToGIDLen = 256;
+  for (i = 0; i < 256; ++i) {
+    codeToGID[i] = 0;
+    if (charProcs && (name = enc[i])) {
+      for (j = 0; j < charProcs->getLength(); j++) {
+	if (strcmp(name, charProcs->getKey(j)) == 0) {
+	  codeToGID[i] = j;
 	}
       }
     }
   }
-  return 1.0;
+
+  return new CairoType3Font(ref, doc, font_face, codeToGID, codeToGIDLen, printing, xref);
 }
 
+CairoType3Font::CairoType3Font(Ref ref,
+			       PDFDoc *doc,
+			       cairo_font_face_t *cairo_font_face,
+			       int *codeToGID,
+			       Guint codeToGIDLen,
+			       GBool printing, XRef *xref) : CairoFont(ref,
+							   cairo_font_face,
+							   codeToGID,
+							   codeToGIDLen,
+							   gFalse,
+							   printing),
+						 doc(doc) { }
+
+CairoType3Font::~CairoType3Font() { }
+
+GBool
+CairoType3Font::matches(Ref &other, GBool printingA) {
+  return (other.num == ref.num && other.gen == ref.gen && printing == printingA);
+}
+
+
 //------------------------------------------------------------------------
 // CairoFontEngine
 //------------------------------------------------------------------------
@@ -537,6 +763,9 @@ CairoFontEngine::CairoFontEngine(FT_Library libA) {
   FT_Library_Version(lib, &major, &minor, &patch);
   useCIDs = major > 2 ||
             (major == 2 && (minor > 1 || (minor == 1 && patch > 7)));
+#if MULTITHREADED
+  gInitMutex(&mutex);
+#endif
 }
 
 CairoFontEngine::~CairoFontEngine() {
@@ -546,26 +775,24 @@ CairoFontEngine::~CairoFontEngine() {
     if (fontCache[i])
       delete fontCache[i];
   }
+#if MULTITHREADED
+  gDestroyMutex(&mutex);
+#endif
 }
 
 CairoFont *
-CairoFontEngine::getFont(GfxFont *gfxFont, XRef *xref) {
+CairoFontEngine::getFont(GfxFont *gfxFont, PDFDoc *doc, GBool printing, XRef *xref) {
   int i, j;
   Ref ref;
   CairoFont *font;
   GfxFontType fontType;
   
-  fontType = gfxFont->getType();
-  if (fontType == fontType3) {
-    /* Need to figure this out later */
-    //    return NULL;
-  }
-
+  fontEngineLocker();
   ref = *gfxFont->getID();
 
   for (i = 0; i < cairoFontCacheSize; ++i) {
     font = fontCache[i];
-    if (font && font->matches(ref)) {
+    if (font && font->matches(ref, printing)) {
       for (j = i; j > 0; --j) {
 	fontCache[j] = fontCache[j-1];
       }
@@ -574,7 +801,12 @@ CairoFontEngine::getFont(GfxFont *gfxFont, XRef *xref) {
     }
   }
   
-  font = CairoFont::create (gfxFont, xref, lib, useCIDs);
+  fontType = gfxFont->getType();
+  if (fontType == fontType3)
+    font = CairoType3Font::create (gfxFont, doc, this, printing, xref);
+  else
+    font = CairoFreeTypeFont::create (gfxFont, xref, lib, useCIDs);
+
   //XXX: if font is null should we still insert it into the cache?
   if (fontCache[cairoFontCacheSize - 1]) {
     delete fontCache[cairoFontCacheSize - 1];
@@ -585,4 +817,3 @@ CairoFontEngine::getFont(GfxFont *gfxFont, XRef *xref) {
   fontCache[0] = font;
   return font;
 }
-
