@@ -13,10 +13,11 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2006-2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006-2010, 2012 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2006 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
 // Copyright (C) 2010 Carlos Garcia Campos <carlosgc@gnome.org>
-// Copyright (C) 2012 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2012, 2013 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -62,6 +63,7 @@ static const char specialChars[256] = {
 };
 
 static const int IntegerSafeLimit = (INT_MAX - 9) / 10;
+static const long long LongLongSafeLimit = (LLONG_MAX - 9) / 10;
 
 //------------------------------------------------------------------------
 // Lexer
@@ -155,10 +157,10 @@ int Lexer::lookChar() {
 Object *Lexer::getObj(Object *obj, int objNum) {
   char *p;
   int c, c2;
-  GBool comment, neg, done, overflownInteger, overflownUnsignedInteger;
+  GBool comment, neg, done, overflownInteger, overflownLongLong;
   int numParen;
   int xi;
-  unsigned int xui = 0;
+  long long xll = 0;
   double xf = 0, scale;
   GooString *s;
   int n, m;
@@ -187,7 +189,7 @@ Object *Lexer::getObj(Object *obj, int objNum) {
   case '5': case '6': case '7': case '8': case '9':
   case '+': case '-': case '.':
     overflownInteger = gFalse;
-    overflownUnsignedInteger = gFalse;
+    overflownLongLong = gFalse;
     neg = gFalse;
     xi = 0;
     if (c == '-') {
@@ -201,23 +203,21 @@ Object *Lexer::getObj(Object *obj, int objNum) {
       c = lookChar();
       if (isdigit(c)) {
 	getChar();
-	if (unlikely(overflownInteger)) {
-	  if (overflownUnsignedInteger) {
-	    xf = xf * 10.0 + (c - '0');
+	if (unlikely(overflownLongLong)) {
+	  xf = xf * 10.0 + (c - '0');
+	} else if (unlikely (overflownInteger)) {
+	  if (unlikely(xll > LongLongSafeLimit) &&
+	      (xll > (LLONG_MAX - (c - '0')) / 10.0)) {
+	    overflownLongLong = gTrue;
+	    xf = xll * 10.0 + (c - '0');
 	  } else {
-	    overflownUnsignedInteger = gTrue;
-	    xf = xui * 10.0 + (c - '0');
+	    xll = xll * 10 + (c - '0');
 	  }
 	} else {
 	  if (unlikely(xi > IntegerSafeLimit) &&
 	      (xi > (INT_MAX - (c - '0')) / 10.0)) {
 	    overflownInteger = gTrue;
-	    if (xi > (UINT_MAX - (c - '0')) / 10.0) {
-	      overflownUnsignedInteger = gTrue;
-	      xf = xi * 10.0 + (c - '0');
-	    } else {
-	      xui = xi * 10.0 + (c - '0');
-	    }
+	    xll = xi * 10LL + (c - '0');
 	  } else {
 	    xi = xi * 10 + (c - '0');
 	  }
@@ -231,13 +231,18 @@ Object *Lexer::getObj(Object *obj, int objNum) {
     }
     if (neg) {
       xi = -xi;
+      xll = -xll;
       xf = -xf;
     }
     if (unlikely(overflownInteger)) {
-      if (overflownUnsignedInteger) {
+      if (overflownLongLong) {
         obj->initReal(xf);
       } else {
-        obj->initUint(xui);
+        if (unlikely(xll == INT_MIN)) {
+          obj->initInt(INT_MIN);
+        } else {
+          obj->initInt64(xll);
+        }
       }
     } else {
       obj->initInt(xi);
@@ -246,8 +251,8 @@ Object *Lexer::getObj(Object *obj, int objNum) {
   doReal:
     if (likely(!overflownInteger)) {
       xf = xi;
-    } else if (!overflownUnsignedInteger) {
-      xf = xui;
+    } else if (!overflownLongLong) {
+      xf = xll;
     }
     scale = 0.1;
     while (1) {
@@ -576,6 +581,48 @@ Object *Lexer::getObj(Object *obj, int objNum) {
   return obj;
 }
 
+Object *Lexer::getObj(Object *obj, const char *cmdA) {
+  char *p;
+  int c;
+  GBool comment;
+  int n;
+
+  // skip whitespace and comments
+  comment = gFalse;
+  const char *cmd1 = tokBuf;
+  *tokBuf = 0;
+  while (strcmp(cmdA, cmd1)) {
+    while (1) {
+      if ((c = getChar()) == EOF) {
+        return obj->initEOF();
+      }
+      if (comment) {
+        if (c == '\r' || c == '\n') {
+          comment = gFalse;
+        }
+      } else if (c == '%') {
+        comment = gTrue;
+      } else if (specialChars[c] != 1) {
+        break;
+      }
+    }
+    p = tokBuf;
+    *p++ = c;
+    n = 1;
+    while ((c = lookChar()) != EOF && specialChars[c] == 0) {
+      getChar();
+      if (++n == tokBufSize) {
+        break;
+      }
+      *p++ = c;
+    }
+    *p = '\0';
+  }
+  obj->initCmd(tokBuf);
+  
+  return obj;
+}
+
 void Lexer::skipToNextLine() {
   int c;
 
