@@ -16,8 +16,13 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2006 Dominic Lachowicz <cinamod@hotmail.com>
-// Copyright (C) 2007-2008 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2008, 2010, 2011 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2009 Jan Jockusch <jan@jockusch.de>
+// Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
+// Copyright (C) 2010 Kenneth Berland <ken@hero.com>
+// Copyright (C) 2011 Tom Gleason <tom@buildadam.com>
+// Copyright (C) 2011 Steven Murdoch <Steven.Murdoch@cl.cam.ac.uk>
+// Copyright (C) 2013 Yury G. Kudryashov <urkud.urkud@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -43,14 +48,17 @@
 #include "Catalog.h"
 #include "Page.h"
 #include "PDFDoc.h"
+#include "PDFDocFactory.h"
 #include "TextOutputDev.h"
 #include "CharTypes.h"
 #include "UnicodeMap.h"
+#include "PDFDocEncoding.h"
 #include "Error.h"
+#include <string>
 
-static void printInfoString(FILE *f, Dict *infoDict, char *key,
-			    char *text1, char *text2, UnicodeMap *uMap);
-static void printInfoDate(FILE *f, Dict *infoDict, char *key, char *fmt);
+static void printInfoString(FILE *f, Dict *infoDict, const char *key,
+			    const char *text1, const char *text2, UnicodeMap *uMap);
+static void printInfoDate(FILE *f, Dict *infoDict, const char *key, const char *fmt);
 
 static int firstPage = 1;
 static int lastPage = 0;
@@ -59,7 +67,9 @@ static int x = 0;
 static int y = 0;
 static int w = 0;
 static int h = 0;
+static GBool bbox = gFalse;
 static GBool physLayout = gFalse;
+static double fixedPitch = 0;
 static GBool rawOrder = gFalse;
 static GBool htmlMeta = gFalse;
 static char textEncName[128] = "";
@@ -89,6 +99,8 @@ static const ArgDesc argDesc[] = {
    "height of crop area in pixels (default is 0)"},
   {"-layout",  argFlag,     &physLayout,    0,
    "maintain original physical layout"},
+  {"-fixed",   argFP,       &fixedPitch,    0,
+   "assume fixed-pitch (or tabular) text"},
   {"-raw",     argFlag,     &rawOrder,      0,
    "keep strings in content stream order"},
   {"-htmlmeta", argFlag,   &htmlMeta,       0,
@@ -101,6 +113,8 @@ static const ArgDesc argDesc[] = {
    "output end-of-line convention (unix, dos, or mac)"},
   {"-nopgbrk", argFlag,     &noPageBreaks,  0,
    "don't insert page breaks between pages"},
+  {"-bbox", argFlag,     &bbox,  0,
+   "output bounding box for each word and page size to html.  Sets -htmlmeta"},
   {"-opw",     argString,   ownerPassword,  sizeof(ownerPassword),
    "owner password (for encrypted files)"},
   {"-upw",     argString,   userPassword,   sizeof(userPassword),
@@ -120,6 +134,30 @@ static const ArgDesc argDesc[] = {
   {NULL}
 };
 
+static std::string myStringReplace(const std::string &inString, const std::string &oldToken, const std::string &newToken) {
+  std::string result = inString;
+  size_t foundLoc;
+  int advance = 0;
+  do {
+    foundLoc = result.find(oldToken, advance);
+    if (foundLoc != std::string::npos){
+      result.replace(foundLoc, oldToken.length(), newToken);
+      advance = foundLoc + newToken.length();
+    }
+  } while (foundLoc != std::string::npos );
+  return result;
+}
+
+static std::string myXmlTokenReplace(const char *inString){
+  std::string myString(inString);
+  myString = myStringReplace(myString, "&",  "&amp;" );
+  myString = myStringReplace(myString, "'",  "&apos;" );
+  myString = myStringReplace(myString, "\"", "&quot;" );
+  myString = myStringReplace(myString, "<",  "&lt;" );
+  myString = myStringReplace(myString, ">",  "&gt;" );
+  return myString;
+}
+
 int main(int argc, char *argv[]) {
   PDFDoc *doc;
   GooString *fileName;
@@ -137,6 +175,9 @@ int main(int argc, char *argv[]) {
 
   // parse args
   ok = parseArgs(argDesc, &argc, argv);
+  if (bbox) {
+    htmlMeta = gTrue;
+  }
   if (!ok || (argc < 2 && !printEnc) || argc > 3 || printVersion || printHelp) {
     fprintf(stderr, "pdftotext version %s\n", PACKAGE_VERSION);
     fprintf(stderr, "%s\n", popplerCopyright);
@@ -144,6 +185,8 @@ int main(int argc, char *argv[]) {
     if (!printVersion) {
       printUsage("pdftotext", "<PDF-file> [<text-file>]", argDesc);
     }
+    if (printVersion || printHelp)
+      exitCode = 0;
     goto err0;
   }
 
@@ -153,10 +196,14 @@ int main(int argc, char *argv[]) {
   if (printEnc) {
     printEncodings();
     delete globalParams;
+    exitCode = 0;
     goto err0;
   }
 
   fileName = new GooString(argv[1]);
+  if (fixedPitch) {
+    physLayout = gTrue;
+  }
 
   if (textEncName[0]) {
     globalParams->setTextEncoding(textEncName);
@@ -175,7 +222,7 @@ int main(int argc, char *argv[]) {
 
   // get mapping to output encoding
   if (!(uMap = globalParams->getTextEncoding())) {
-    error(-1, "Couldn't get text encoding");
+    error(errCommandLine, -1, "Couldn't get text encoding");
     delete fileName;
     goto err1;
   }
@@ -192,15 +239,13 @@ int main(int argc, char *argv[]) {
     userPW = NULL;
   }
 
-  if(fileName->cmp("-") != 0) {
-      doc = new PDFDoc(fileName, ownerPW, userPW);
-  } else {
-      Object obj;
-
-      obj.initNull();
-      doc = new PDFDoc(new FileStream(stdin, 0, gFalse, 0, &obj), ownerPW, userPW);
+  if (fileName->cmp("-") == 0) {
+      delete fileName;
+      fileName = new GooString("fd://0");
   }
 
+  doc = PDFDocFactory().createPDFDoc(*fileName, ownerPW, userPW);
+
   if (userPW) {
     delete userPW;
   }
@@ -215,7 +260,7 @@ int main(int argc, char *argv[]) {
 #ifdef ENFORCE_PERMISSIONS
   // check for copy permission
   if (!doc->okToCopy()) {
-    error(-1, "Copying of text from this document is not allowed.");
+    error(errNotAllowed, -1, "Copying of text from this document is not allowed.");
     exitCode = 3;
     goto err2;
   }
@@ -224,6 +269,9 @@ int main(int argc, char *argv[]) {
   // construct text file name
   if (argc == 3) {
     textFileName = new GooString(argv[2]);
+  } else if (fileName->cmp("fd://0") == 0) {
+     error(errCommandLine, -1, "You have to provide an output filename when reading form stdin.");
+     goto err2;
   } else {
     p = fileName->getCString() + fileName->getLength() - 4;
     if (!strcmp(p, ".pdf") || !strcmp(p, ".PDF")) {
@@ -249,62 +297,98 @@ int main(int argc, char *argv[]) {
       f = stdout;
     } else {
       if (!(f = fopen(textFileName->getCString(), "wb"))) {
-	error(-1, "Couldn't open text file '%s'", textFileName->getCString());
+	error(errIO, -1, "Couldn't open text file '{0:t}'", textFileName);
 	exitCode = 2;
 	goto err3;
       }
     }
-    fputs("<html>\n", f);
+    fputs("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">", f);
+    fputs("<html xmlns=\"http://www.w3.org/1999/xhtml\">\n", f);
     fputs("<head>\n", f);
     doc->getDocInfo(&info);
     if (info.isDict()) {
-      printInfoString(f, info.getDict(), "Title", "<title>", "</title>\n",
-		      uMap);
+      Object obj;
+      if (info.getDict()->lookup("Title", &obj)->isString()) {
+        printInfoString(f, info.getDict(), "Title", "<title>", "</title>\n", uMap);
+      } else {
+        fputs("<title></title>\n", f);
+      }
+      obj.free();
       printInfoString(f, info.getDict(), "Subject",
-		      "<meta name=\"Subject\" content=\"", "\">\n", uMap);
+		      "<meta name=\"Subject\" content=\"", "\"/>\n", uMap);
       printInfoString(f, info.getDict(), "Keywords",
-		      "<meta name=\"Keywords\" content=\"", "\">\n", uMap);
+		      "<meta name=\"Keywords\" content=\"", "\"/>\n", uMap);
       printInfoString(f, info.getDict(), "Author",
-		      "<meta name=\"Author\" content=\"", "\">\n", uMap);
+		      "<meta name=\"Author\" content=\"", "\"/>\n", uMap);
       printInfoString(f, info.getDict(), "Creator",
-		      "<meta name=\"Creator\" content=\"", "\">\n", uMap);
+		      "<meta name=\"Creator\" content=\"", "\"/>\n", uMap);
       printInfoString(f, info.getDict(), "Producer",
-		      "<meta name=\"Producer\" content=\"", "\">\n", uMap);
+		      "<meta name=\"Producer\" content=\"", "\"/>\n", uMap);
       printInfoDate(f, info.getDict(), "CreationDate",
-		    "<meta name=\"CreationDate\" content=\"\">\n");
+		    "<meta name=\"CreationDate\" content=\"\"/>\n");
       printInfoDate(f, info.getDict(), "LastModifiedDate",
-		    "<meta name=\"ModDate\" content=\"\">\n");
+		    "<meta name=\"ModDate\" content=\"\"/>\n");
     }
     info.free();
     fputs("</head>\n", f);
     fputs("<body>\n", f);
-    fputs("<pre>\n", f);
-    if (f != stdout) {
-      fclose(f);
+    if (!bbox) {
+      fputs("<pre>\n", f);
+      if (f != stdout) {
+	fclose(f);
+      }
     }
   }
 
   // write text file
-  textOut = new TextOutputDev(textFileName->getCString(),
-			      physLayout, rawOrder, htmlMeta);
-  if (textOut->isOk()) {
-    if ((w==0) && (h==0) && (x==0) && (y==0)) {
-      doc->displayPages(textOut, firstPage, lastPage, resolution, resolution, 0,
-			gTrue, gFalse, gFalse);
-    } else {
-      int page;
-      
-      for (page = firstPage; page <= lastPage; ++page) {
-	doc->displayPageSlice(textOut, page, resolution, resolution, 0,
+  if (bbox) {
+    textOut = new TextOutputDev(NULL, physLayout, fixedPitch, rawOrder, htmlMeta);
+
+    if (textOut->isOk()) {
+      fprintf(f, "<doc>\n");
+      for (int page = firstPage; page <= lastPage; ++page) {
+        fprintf(f, "  <page width=\"%f\" height=\"%f\">\n",doc->getPageMediaWidth(page), doc->getPageMediaHeight(page));
+        doc->displayPage(textOut, page, resolution, resolution, 0, gTrue, gFalse, gFalse);
+        TextWordList *wordlist = textOut->makeWordList();
+        const int word_length = wordlist != NULL ? wordlist->getLength() : 0;
+        TextWord *word;
+        double xMinA, yMinA, xMaxA, yMaxA;
+        if (word_length == 0)
+          fprintf(stderr, "no word list\n");
+
+        for (int i = 0; i < word_length; ++i) {
+          word = wordlist->get(i);
+          word->getBBox(&xMinA, &yMinA, &xMaxA, &yMaxA);
+          const std::string myString = myXmlTokenReplace(word->getText()->getCString());
+          fprintf(f,"    <word xMin=\"%f\" yMin=\"%f\" xMax=\"%f\" yMax=\"%f\">%s</word>\n", xMinA, yMinA, xMaxA, yMaxA, myString.c_str());
+        }
+        fprintf(f, "  </page>\n");
+        delete wordlist;
+      }
+      fprintf(f, "</doc>\n");
+    }
+    fclose(f);
+  } else {
+    textOut = new TextOutputDev(textFileName->getCString(),
+				physLayout, fixedPitch, rawOrder, htmlMeta);
+    if (textOut->isOk()) {
+      if ((w==0) && (h==0) && (x==0) && (y==0)) {
+	doc->displayPages(textOut, firstPage, lastPage, resolution, resolution, 0,
+			  gTrue, gFalse, gFalse);
+      } else {
+	
+	for (int page = firstPage; page <= lastPage; ++page) {
+	  doc->displayPageSlice(textOut, page, resolution, resolution, 0,
 			      gTrue, gFalse, gFalse, 
 			      x, y, w, h);
-      }	
-    }	
+	}
+      }
 
-  } else {
+    } else {
     delete textOut;
     exitCode = 2;
     goto err3;
+    }
   }
   delete textOut;
 
@@ -314,12 +398,12 @@ int main(int argc, char *argv[]) {
       f = stdout;
     } else {
       if (!(f = fopen(textFileName->getCString(), "ab"))) {
-	error(-1, "Couldn't open text file '%s'", textFileName->getCString());
+	error(errIO, -1, "Couldn't open text file '{0:t}'", textFileName);
 	exitCode = 2;
 	goto err3;
       }
     }
-    fputs("</pre>\n", f);
+    if (!bbox) fputs("</pre>\n", f);
     fputs("</body>\n", f);
     fputs("</html>\n", f);
     if (f != stdout) {
@@ -334,6 +418,7 @@ int main(int argc, char *argv[]) {
   delete textFileName;
  err2:
   delete doc;
+  delete fileName;
   uMap->decRefCnt();
  err1:
   delete globalParams;
@@ -346,8 +431,8 @@ int main(int argc, char *argv[]) {
   return exitCode;
 }
 
-static void printInfoString(FILE *f, Dict *infoDict, char *key,
-			    char *text1, char *text2, UnicodeMap *uMap) {
+static void printInfoString(FILE *f, Dict *infoDict, const char *key,
+			    const char *text1, const char *text2, UnicodeMap *uMap) {
   Object obj;
   GooString *s1;
   GBool isUnicode;
@@ -372,7 +457,7 @@ static void printInfoString(FILE *f, Dict *infoDict, char *key,
 	    (s1->getChar(i+1) & 0xff);
 	i += 2;
       } else {
-	u = s1->getChar(i) & 0xff;
+	u = pdfDocEncoding[s1->getChar(i) & 0xff];
 	++i;
       }
       n = uMap->mapUnicode(u, buf, sizeof(buf));
@@ -383,7 +468,7 @@ static void printInfoString(FILE *f, Dict *infoDict, char *key,
   obj.free();
 }
 
-static void printInfoDate(FILE *f, Dict *infoDict, char *key, char *fmt) {
+static void printInfoDate(FILE *f, Dict *infoDict, const char *key, const char *fmt) {
   Object obj;
   char *s;
 
